/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include <pwd.h>
#include <types.h>
#include <jpidef.h>
#include <starlet.h>
#include <stdio.h>
#include <ssdef.h>
#include <stsdef.h>
#include <stdlib.h>
#include <unixio.h>
#include <errno.h>
#include <unistd.h>

typedef struct {short   buflen;
                short   itmcode;
                void    *buffer;
                void    *retlen; } ITMLST;

unsigned long prevpriv[2];
unsigned long allprivs[2] = { 0xFFFFFFFF, 0xFFFFFFFF };

/*
 * A procedure to remove any privs that were obtained purely because the 
 * image was installed with them.
 */

int GENERIC_REMOVE_IMAGE_PRIVS () { 

    int status;
    static short retlen;
    static unsigned long procpriv[2];
    static ITMLST item[2] ={ {12, JPI$_PROCPRIV, procpriv, &retlen},
                       {0, 0, 0, 0} };
    unsigned long mypid = getpid();;


/*
 Get just the process privs (without the image privs).
*/
    status=sys$getjpiw(0, &mypid, 0, item, 0, 0 ,0);
    if ((status & 1) == 0)
	return(status);

/*
 Turn off all privs and then turn on just the process privs
*/
    status = sys$setprv(0,allprivs,0,0);
    if ((status & 1) == 0)
	return(status);
    status = sys$setprv(1,procpriv,0,0);
    if ((status & 1) == 0)
	return(status);

    return(1);
}


/*
 * Procedure to set the specified privs. The previous privs are
 * saved and can be restored via a call to VMS_restore_privs.
 * The two params are the two longwords of the priv mask.
 */

int GENERIC_SET_PRIVS (unsigned long m1, unsigned long m2) { 
    unsigned long privs[2];
    privs[0]=m1;
    privs[1]=m2;
    return(sys$setprv(1,privs,0,prevpriv));
}

int GENERIC_RESTORE_PRIVS () { 
    unsigned long status;
    status = sys$setprv(0,allprivs,0,0);
    if ((status & 1) == 0)
	return(status);
    status = sys$setprv(1,prevpriv,0,0);
    if ((status & 1) == 0)
	return(status);
    return(1);
}

