/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include "vms_jackets.h"
#include <file.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stat.h>
#include <errno.h>

/**********************************************************************/
/*  routines to handle VMS and decc limits                            */
/*                                                                    */
/*  Routine: GENERIC_File_Share_REG                                   */
/*  Routine: GENERIC_File_Share_UNREG                                 */
/*  Routine: GENERIC_Mark_Share_WRITE                                 */
/*  Routine: GENERIC_File_Share_CHECK                                 */
/*                                                                    */
/*                                                                    */
/*  Reason:  the only way to get file share support is to sync before */
/*           reading, or sync after every write.  syncing after every */
/*           write is extremely painful and very very very slow       */
/*           Use GENERIC_FILE_SHARE_REG to register a file handle     */
/*           and GENERIC_FILE_SHARE_CHECK to see if it needs to       */
/*           write/sync before reading.                               */
/**********************************************************************/

#define MAX_SHARE_COUNT 1024
typedef struct {
	int fileno;
	__ino_t ino[3];
	int write_pending;
	int write_append_mode;
	int open_mode;}
		GENERIC_file_record ;

GENERIC_file_record GENERIC_SHARED_FILE_ARRAY[MAX_SHARE_COUNT];
int GENERIC_current_max_fileno =0; 

int GENERIC_FILE_SHARE_INIT()
{
    int i;
	memset(GENERIC_SHARED_FILE_ARRAY,0,sizeof(GENERIC_SHARED_FILE_ARRAY));
	for (i=0;i<MAX_SHARE_COUNT;i++)
		GENERIC_SHARED_FILE_ARRAY[i].fileno= -1;/* invalidate it*/
    return 0;
}

int GENERIC_FILE_SHARE_REG(int file_handle, int mode )
{
   int return_int,status,save_errno,save_vmserrno;
   struct stat stat_record;

   save_errno = errno;
   save_vmserrno = vaxc$errno;

   return_int = fstat(file_handle,&stat_record);
   if (return_int == -1)
	{
#ifdef DEBUG
	printf("fstat error in GENERIC_FILE_SHARE_REG\n");
#endif
	errno = save_errno;
	vaxc$errno = save_vmserrno;
	return return_int;
	};
   GENERIC_SHARED_FILE_ARRAY[file_handle].open_mode = mode;
   GENERIC_SHARED_FILE_ARRAY[file_handle].fileno =file_handle;
   GENERIC_SHARED_FILE_ARRAY[file_handle].ino[0] = stat_record.st_ino[0]; 
   GENERIC_SHARED_FILE_ARRAY[file_handle].ino[1] = stat_record.st_ino[1]; 
   GENERIC_SHARED_FILE_ARRAY[file_handle].ino[2] = stat_record.st_ino[2]; 
   GENERIC_SHARED_FILE_ARRAY[file_handle].write_pending=0;
   if (mode &  (O_WRONLY | O_CREAT | O_TRUNC))
	GENERIC_SHARED_FILE_ARRAY[file_handle].write_append_mode =1;
   else
	GENERIC_SHARED_FILE_ARRAY[file_handle].write_append_mode =0;

   errno = save_errno;
   vaxc$errno = save_vmserrno;
   if (GENERIC_current_max_fileno < file_handle)
	GENERIC_current_max_fileno = file_handle;

   return 0;

}

int GENERIC_FILE_SHARE_UNREG(int file_handle)
{
   GENERIC_SHARED_FILE_ARRAY[file_handle].fileno= -1;
   return 0;
}

int GENERIC_MARK_SHARE_WRITE(int file_handle)
{
   GENERIC_SHARED_FILE_ARRAY[file_handle].write_pending = 1;
   return 0;
}

int GENERIC_FILE_SHARE_CHECK(int file_handle)
{
   int i,ret_status;
   ret_status =0;
   for (i=0;i<=GENERIC_current_max_fileno;i++)
	if ((GENERIC_SHARED_FILE_ARRAY[i].fileno != -1) &&
		GENERIC_SHARED_FILE_ARRAY[i].write_pending)
		if ((GENERIC_SHARED_FILE_ARRAY[i].ino[0]==
			GENERIC_SHARED_FILE_ARRAY[file_handle].ino[0]) &&
		  (GENERIC_SHARED_FILE_ARRAY[i].ino[1]==
			GENERIC_SHARED_FILE_ARRAY[file_handle].ino[1]) &&
		  (GENERIC_SHARED_FILE_ARRAY[i].ino[2]==
			GENERIC_SHARED_FILE_ARRAY[file_handle].ino[2]))
			{
			GENERIC_SHARED_FILE_ARRAY[i].write_pending=0;
			fsync(GENERIC_SHARED_FILE_ARRAY[i].fileno);
			ret_status =1;
			};
   return ret_status;
}
