/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include "vms_jackets.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

/**********************************************************************/
/*  Jacket routines to handle VMS and decc limits                     */
/*                                                                    */
/*  Routine: GENERIC_TEMPNAM_JACKET                                   */
/*                                                                    */
/*  Jacket for: tempnam                                               */
/*                                                                    */
/*  Reason:  tempnam if passed a "/tmp" for the directory returns an  */
/*           invalid file spec                                        */
/*  Fix:     if "/tmp" is found use "/sys$scratch"                    */
/*                                                                    */
/*                                                                    */
/**********************************************************************/

static char* make_thread_specific_string(char* first_string)
{
pthread_t id;
char     t[20];   /* more than enough for number of digits in an int */
char*    new_string;

   id= pthread_self();      
   sprintf(t,"_%u",(int)id);
   
   new_string= (char *)malloc(strlen(first_string) + strlen(t) + 1);
   strcpy(new_string,first_string);
   strcat(new_string,t);
   free(first_string);
   return new_string;
}

char *GENERIC_TEMPNAM_JACKET(char *directory, char *prefix)
{

   char *return_string;
   /* let check for just "/tmp", if so make it "/sys$scratch/" */
   
   if (!strcmp(directory,"/tmp"))
	return_string = tempnam("/sys$scratch/",prefix);
   else
      return_string = tempnam(directory,prefix);

   return(make_thread_specific_string(return_string));
}
