/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


/*****************************************************************************/
/*                                                                           */
/* Jacket routines for localtime_r and gmtime_r                              */
/*                                                                           */
/* Reason:  VMS doesn't have a localtime_r or gmtime_r                       */
/*                                                                           */
/*****************************************************************************/
/*
** As of DECC 5.7 (VMS V7.2) these two routines are available, but it
** doesn't (well shouldn't) do any harm to always use these.
*/

#include <string.h>
#include <time.h>

struct tm *GENERIC_LOCALTIME_R (const time_t *timer, struct tm *result) {
   struct tm *local_result = localtime(timer);
   return (memcpy (result, local_result, sizeof(struct tm)));
}

struct tm *GENERIC_GMTIME_R (const time_t *timer, struct tm *result) {
   struct tm *local_result = gmtime(timer);
   return (memcpy (result, local_result, sizeof(struct tm)));
}

/*
** This is messy, but what can we do.
**
** As of V7.2, time.h prototypes localtime_r and gmtime_r with a __UTCTZ_
** prefix pragma. Since we have these defined to be GENERIC_whatever the
** compiler/linker between themselves end up converting a stat call to
** a call to __UTCTZ_GENERIC_whatever. The simplest way to fix this
** is to give it such a routine, that does nothing except call our jacket.
*/
struct tm *__UTCTZ_GENERIC_LOCALTIME_R (const time_t *timer, struct tm *result) {
    return GENERIC_LOCALTIME_R (timer, result);
}

struct tm *__UTCTZ_GENERIC_GMTIME_R (const time_t *timer, struct tm *result) {
    return GENERIC_GMTIME_R (timer, result);
}

