/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


/*
**  MODULE DESCRIPTION: a utime function for VMS
*/

#include "vms_jackets.h"
#include <stdio.h>
#include <string.h>
#include <fibdef.h>
#include <atrdef.h>
#include <sbkdef.h>
#include <iodef.h>
#include <rms.h>
#include <descrip.h>
#include <starlet.h>
#include <lib$routines.h>
#include <stat.h>
#include <unixlib.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

/* suspect utime.h from POSIX */
#include "utime.h"
/***************************/
/********** utime **********/
/***************************/
int GENERIC_UTIME (const char *file_spec, const struct utimbuf *times)
{
   unsigned int base_time[2], temp[2], unix_timecheck;
   unsigned long unix_time;
   char tracebuf[128];
   struct tm *stupid_tm;

   struct FAB    fab;           /* File attributes block                   */
   struct NAM    nam;           /* Name block                              */
   struct fibdef fib;           /* File information block                  */

  struct FIB_DESC
                {
                   int length;          /* Length of file information block  */
                   struct fibdef *fib;  /* Address of file information block */
                }
                fib_desc =
                {
                  FIB$C_LENGTH,
                  &fib
                };

   unsigned int revdate[2];                /* 64 bit create date */

   struct atrdef  atr[2] =           /* File attributes block               */
                  {
#ifdef __ALPHA
                     {ATR$S_REVDATE, ATR$C_REVDATE, &revdate}, /* date    */
#else
/* For some reason the prototype is wrong on VAX - should be void * not uint */
                     {ATR$S_REVDATE, ATR$C_REVDATE, (unsigned int) &revdate}, /* date    */
#endif
                     {0,0,0}                               /* End of atr    */
                  };

   char full_name[NAM$C_MAXRSS]; /* Full filename returned by RMS           */
   char nam_esa[NAM$C_MAXRSS];   /* Needed by search and parse              */
   $DESCRIPTOR(fullname_desc, full_name);  /* Needed by $ASSIGN             */

   /* Used by $QIO system service... */
   unsigned long iosb[2], status, function;
   unsigned short channel;

   /*
   ** Somehow I just don't think UNIX filespecs are going to cut it,
   ** when we start doing this system service calls.
   */
   char temp_string[TEMP_STRING_SIZE];
   char *new_filename, *newVMS_filename;
   int  fnp_status;

   new_filename = GENERIC_VMS_Filename_parser((char *)file_spec,
                                        temp_string,
                                        GENERIC_K_STRING_TYPE_FILE,
                                        &fnp_status);
   newVMS_filename = GENERIC_EXTERNAL_NAME( new_filename, 0 );
   GENERIC_TRACE("utime: Mapped:", file_spec, "To:", newVMS_filename);

   /* Initialise the fab block  */
   fab = cc$rms_fab;
   fab.fab$l_nam = &nam;
   fab.fab$l_fop = FAB$M_NAM;
   fab.fab$l_fna = (char *)newVMS_filename;
   fab.fab$b_fns = strlen(newVMS_filename);

   /* Initialise the nam block */
   nam = cc$rms_nam;
   nam.nam$l_rsa = (char *)&full_name; /* Full filename wanted at this addr  */
   nam.nam$b_rss = NAM$C_MAXRSS;    /* maximum size of full filename         */
   nam.nam$l_esa = (char *)&nam_esa;/* Expanded string address               */
   nam.nam$b_ess = NAM$C_MAXRSS;    /* maximum size of expanded filename str */



   /* Use RMS to find the file and fill in the NAM and FAB blocks       */
   if ( ((status=sys$parse(&fab,0,0)) & 1) == 1)
       status = sys$search(&fab,0,0);

   if ((status & 1) == 0) return status;


   /* Initialise the fib block  */
   memset(&fib,0,FIB$C_LENGTH);

   /* Copy the File ID from the nam block to the fib block */
    memcpy( &fib.fib$r_fid_overlay , &nam.nam$w_fid[0], 6 );

   /* Fill in the length of the filename in the descriptor */
   fullname_desc.dsc$w_length = nam.nam$b_rsl;

   /* Assign a channel to the file so it can be opened     */

    status = sys$assign(
                        &fullname_desc, /* Device Name             */
                        &channel,  /* Returned Channel number */
                        0,              /* Access Mode             */
                        0               /* Mailbox Name            */
                       );

   if ((status & 1) == 0) return status;

   /*************************************************************************
    * We must now issue an ACCESS QIO to open the file.  We need to specify *
    * a file attributes block and a statistics block in order to obtain     *
    * the allocation count for the file                                     *
    *************************************************************************/

    /* Say that we intend to write */
    fib.fib$l_acctl = FIB$M_WRITE;

   function = IO$_ACCESS | IO$M_ACCESS;
   status = sys$qiow(
           0,                     /* Event Flag Number         */
           channel,               /* I/O channel               */
           function,              /* I/O function code         */
           iosb,                  /* I/O status block address  */
           0,                     /* AST Address               */
           0,                     /* AST Parameter             */
           &fib_desc,             /* P1, fib descriptor        */
           0,                     /* P2, not used              */
           0,                     /* P3, not used              */
           0,                     /* P4, not used              */
           atr,                   /* P5, Attributes block      */
           0);                    /* P6, not used              */

   if ((status & 1) == 0) return status;
   if ((iosb[0] & 1) == 0) return iosb[0];

   /*
   ** If times == NULL then we set modtime to current time.
   ** Note: on VMS actime = modtime (since actime doesn't really apply.)
   */
   if ( times )
      {
      unix_time = times->modtime;

      /*
      ** For some crazy reason, when we feed the VMS time to the QIO
      ** it gets modified into GMT !!!  This is darn darn annoying,
      ** so I going to attempt to use tm_gmtoff to compensate.
      ** NOTE: This aint ANSI89!
      */
      stupid_tm = localtime( &unix_time );
      if ( stupid_tm->tm_gmtoff ) unix_time += stupid_tm->tm_gmtoff;

      /*
      ** base_time is the length of time in 100 nanosecond intervals
      ** between 11/17/1885 & 1/1/1970 which we need to calculate unix time
      ** from VMS time (reverse of decc$fix_time function)
      */
      base_time[0] = 0x4beb4000;
      base_time[1] = 0x007c9567;
      lib$emul( &10000000, &unix_time, &0, temp );
      lib$addx( temp, base_time, revdate, &2 );

      unix_timecheck = decc$fix_time( revdate );

      sprintf(tracebuf, "utime( %s, %d )\nunix_timecheck=%d",
              newVMS_filename, unix_time, unix_timecheck );
      }
   else
      {
      sys$gettim( &revdate[0] );
      sprintf(tracebuf, "utime( %s, 0 )", newVMS_filename );
      }
   GENERIC_TRACE("VMS:", tracebuf, "", "");

   /* Now update the creation date */

   function = IO$_DEACCESS;
   status = sys$qiow(
           0,                     /* Event Flag Number         */
           channel,               /* I/O channel               */
           function,              /* I/O function code         */
           iosb,                  /* I/O status block address  */
           0,                     /* AST Address               */
           0,                     /* AST Parameter             */
           &fib_desc,             /* P1, fib descriptor        */
           0,                     /* P2, not used              */
           0,                     /* P3, not used              */
           0,                     /* P4, not used              */
           atr,                   /* P5, Attributes block      */
           0);                    /* P6, not used              */

   if ((status & 1) == 0) return status;
   if ((iosb[0] & 1) == 0) return iosb[0];

   status = sys$dassgn(
                        channel  /* Channel number */
                       );

   if ((status & 1) == 0) return status;

   return 1;
}
